/**
 * The MySensors Arduino library handles the wireless radio link and protocol
 * between your home built sensors/actuators and HA controller of choice.
 * The sensors forms a self healing radio network with optional repeaters. Each
 * repeater and gateway builds a routing tables in EEPROM which keeps track of the
 * network topology allowing messages to be routed to nodes.
 *
 * Created by Henrik Ekblad <henrik.ekblad@mysensors.org>
 * Copyright (C) 2013-2015 Sensnology AB
 * Full contributor list: https://github.com/mysensors/Arduino/graphs/contributors
 *
 * Documentation: http://www.mysensors.org
 * Support Forum: http://forum.mysensors.org
 * 
 * * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * Weather forecasting lamp for MySensors
 * Creator: Gunnar Blockmar 2019-11-15
 *******************************
 *
 * REVISION HISTORY
 * 1.0 
 *   First effort
 * 2.0
 *   Added night mode that does not keep people awake 
 * 
 */

// #define MY_OTA_FIRMWARE_FEATURE
// #define MY_REPEATER_FEATURE
#define MY_NODE_ID 70

#define MY_RADIO_NRF24
// #define MY_RF24_PA_LEVEL  RF24_PA_MAX

#define MY_DEBUG
// #define MY_DEBUG_VERBOSE



#include <FastLED.h>
FASTLED_USING_NAMESPACE

#include <MySensors.h>

#define CHILD_ID_FORECAST 0

#define SN "Drömsyn Weather"
#define SV "2.0.0"

#if defined(FASTLED_VERSION) && (FASTLED_VERSION < 3001000)
#warning "Requires FastLED 3.1 or later; check github for latest code."
#endif

#define DATA_PIN    5
//#define CLK_PIN   4
#define LED_TYPE    WS2812
#define COLOR_ORDER GRB
#define NUM_LEDS    32
CRGB leds[NUM_LEDS];  // Set up array that holds info for each LED

#define BRIGHTNESS         176
#define FRAMES_PER_SECOND  120

String color;

CRGBPalette16 currentPalette;
TBlendType    currentBlending;

// The various palettes for different weather types are declared here and stored in PROGMEM

extern CRGBPalette16 ThunderStorm;
extern const TProgmemPalette16 ThunderStorm_p PROGMEM;

extern CRGBPalette16 Drizzle;
extern const TProgmemPalette16 Drizzle_p PROGMEM;

extern CRGBPalette16 Rain;
extern const TProgmemPalette16 Rain_p PROGMEM;

extern CRGBPalette16 Snow;
extern const TProgmemPalette16 Snow_p PROGMEM;

extern CRGBPalette16 Fog;
extern const TProgmemPalette16 Fog_p PROGMEM;

extern CRGBPalette16 Clear;
extern const TProgmemPalette16 Clear_p PROGMEM;

extern CRGBPalette16 Cloud;
extern const TProgmemPalette16 Cloud_p PROGMEM;

extern CRGBPalette16 Night;
extern const TProgmemPalette16 Night_p PROGMEM;

int colorIndex;
int colorChange;
int val;
int addGlitter;


MyMessage forecastMsg(CHILD_ID_FORECAST, V_VAR1);

void setup() {
  delay(3000); // 3 second delay for recovery
  
  // tell FastLED about the LED strip configuration
  FastLED.addLeds<LED_TYPE,DATA_PIN,COLOR_ORDER>(leds, NUM_LEDS).setCorrection(TypicalLEDStrip);
  // set master brightness control
  FastLED.setBrightness(BRIGHTNESS);

  currentPalette = RainbowColors_p;  // Palette used at startup as self-test until first forecast message arrives
  currentBlending = LINEARBLEND;
  colorChange = 1;

  
}

void presentation()
{
  // Send the Sketch Version Information to the Gateway
  sendSketchInfo(SN, SV);
  present(CHILD_ID_FORECAST, S_CUSTOM);
}



void loop()  {
  
    static uint8_t startIndex = 0;
    startIndex = startIndex - 2; // Sets the speed of colors walking through the LED matrix
    
    FillLEDsFromPaletteColors( startIndex );
    
    if (val >= 200 and val < 299 ) {  // Adds flashes if the forecast is lightning
      addGlitter = (val - 200) * 4 + 16;
      addLightning(addGlitter);
    }
    
    if (val >= 300 and val < 399 ) {  // Adds a few raindrops if the forecast is drizzle
      addGlitter = (val - 300) * 12 + 16;
      addRain(addGlitter);
    }
    
    if (val >= 500 and val < 599 ) {  // Adds more raindrops if the forecast is rain
      addGlitter = (val - 500) * 40 + 16;
      addRain(addGlitter);
    }
    
    if (val >= 600 and val < 699 ) {  // Adds snowflakes if the forecast is snow
      addGlitter = (val - 600) * 40 + 16;
      addSnow(addGlitter);
    }
    
    FastLED.show();
    colorChange++;  
  // insert a delay to keep the framerate modest
    FastLED.delay(10000/FRAMES_PER_SECOND);
}
  


void receive(const MyMessage &message) {  // Receive the forecast and select the corresponding palette
  if (message.type == V_VAR1) {
    val = atoi(message.data);
    if (val >= 200 and val < 299 ) currentPalette = ThunderStorm_p;
    if (val >= 300 and val < 399 ) currentPalette = Drizzle_p;
    if (val >= 500 and val < 599 ) currentPalette = Rain_p;
    if (val >= 600 and val < 699 ) currentPalette = Fog_p;
    if (val >= 700 and val < 799 ) currentPalette = Fog_p;
    if (val == 800 )               currentPalette = Clear_p;
    if (val >= 801 and val < 805 ) currentPalette = Cloud_p;
    if (val == 1000)               currentPalette = Night_p;
  } 
  else {
    return;  // Do nothing if forecast number is out of range or the message is the wrong type
  }

}

void FillLEDsFromPaletteColors( uint8_t colorIndex) {  // Fill the LED array with colors from the selected palette
    uint8_t brightness = BRIGHTNESS;
    
    for( int i = 0; i < NUM_LEDS; i++) {
        leds[i] = ColorFromPalette( currentPalette, colorIndex, brightness, currentBlending);
        colorIndex += 1;
    }
}



void addRain( fract8 chanceOfRain) {  // Add random flashes of blue
  if( random8() < chanceOfRain) {
    leds[ random16(NUM_LEDS) ] += CRGB::Blue;
  }
}

void addLightning( fract8 chanceOfLightning) {  // Flash all LEDs white as lightning
  if( random8() < chanceOfLightning) {
    for (int i = 0; i < NUM_LEDS; i++)  {
      leds[ i ] += CRGB::White;
    }
  }
}

void addSnow( fract8 chanceOfSnow) {  // Add random flashes of white
  if( random8() < chanceOfSnow) {
    leds[ random16(NUM_LEDS) ] += CRGB::White;
  }
}


// Below, all palettes are defined using the standardized HTML color names 

const TProgmemPalette16 ThunderStorm_p PROGMEM =
{
    CRGB::BlueViolet,
    CRGB::BlueViolet,
    CRGB::DarkSlateGray,
    CRGB::DarkSlateGray,
    
    CRGB::BlueViolet,
    CRGB::BlueViolet,
    CRGB::DarkSlateGray,
    CRGB::DarkSlateGray,
    
    CRGB::BlueViolet,
    CRGB::BlueViolet,
    CRGB::DarkSlateGray,
    CRGB::DarkSlateGray,
    
    CRGB::BlueViolet,
    CRGB::BlueViolet,
    CRGB::DarkSlateGray,
    CRGB::DarkSlateGray
};

const TProgmemPalette16 Drizzle_p PROGMEM =
{
    CRGB::Blue,
    CRGB::Blue, 
    CRGB::Gray,
    CRGB::Gray,
    
    CRGB::Blue,
    CRGB::Blue, 
    CRGB::Gray,
    CRGB::Gray,
    
    CRGB::Blue,
    CRGB::Blue, 
    CRGB::Gray,
    CRGB::Gray,
    
    CRGB::Blue,
    CRGB::Blue, 
    CRGB::Gray,
    CRGB::Gray
};

const TProgmemPalette16 Rain_p PROGMEM =
{
    CRGB::Black,
    CRGB::Black, 
    CRGB::Black,
    CRGB::Black,
    
    CRGB::DarkSlateGray,
    CRGB::DarkSlateGray, 
    CRGB::DarkSlateGray,
    CRGB::DarkSlateGray,
    
    CRGB::Black,
    CRGB::Black, 
    CRGB::Black,
    CRGB::Black,
    
    CRGB::DarkSlateGray,
    CRGB::DarkSlateGray, 
    CRGB::DarkSlateGray,
    CRGB::DarkSlateGray
};

const TProgmemPalette16 Snow_p PROGMEM =
{
    CRGB::Black,
    CRGB::Cyan, 
    CRGB::Black,
    CRGB::Cyan,
    
    CRGB::Blue,
    CRGB::Cyan, 
    CRGB::Black,
    CRGB::Cyan,
    
    CRGB::Black,
    CRGB::Cyan, 
    CRGB::Black,
    CRGB::Cyan,
    
    CRGB::Blue,
    CRGB::Cyan, 
    CRGB::Black,
    CRGB::Cyan
};

const TProgmemPalette16 Fog_p PROGMEM =
{
    CRGB::Gray,
    CRGB::Black,
    CRGB::Gray,
    CRGB::Black,
    
    CRGB::Gray,
    CRGB::Black,
    CRGB::Gray,
    CRGB::Black,
    
    CRGB::Gray,
    CRGB::Black,
    CRGB::Gray,
    CRGB::Black,
    
    CRGB::Gray,
    CRGB::Black,
    CRGB::Gray,
    CRGB::Black
};

const TProgmemPalette16 Clear_p PROGMEM =
{
    CRGB::Yellow,
    CRGB::Yellow, 
    CRGB::Yellow,
    CRGB::Yellow,
    
    CRGB::Blue,
    CRGB::Blue,
    CRGB::Blue,
    CRGB::Blue,
    
    CRGB::Blue,
    CRGB::Blue,
    CRGB::Blue,
    CRGB::Blue,
    
    CRGB::Blue,
    CRGB::Blue,
    CRGB::Blue,
    CRGB::Blue
};

const TProgmemPalette16 Cloud_p PROGMEM =
{
    CRGB::White,
    CRGB::Yellow, 
    CRGB::Yellow,
    CRGB::Yellow,
    
    CRGB::White,
    CRGB::White,
    CRGB::Blue,
    CRGB::Blue,
    
    CRGB::White,
    CRGB::White,
    CRGB::White,
    CRGB::Blue,
    
    CRGB::Blue,
    CRGB::Blue,
    CRGB::White,
    CRGB::White
};

const TProgmemPalette16 Night_p PROGMEM =
{
    CRGB::Black,
    CRGB::Black, 
    CRGB::Black,
    CRGB::Black,
    
    CRGB::Black,
    CRGB::Black, 
    CRGB::Black,
    CRGB::Black,
    
    CRGB::Black,
    CRGB::Black, 
    CRGB::Black,
    CRGB::Black,
    
    CRGB::Black,
    CRGB::Black, 
    CRGB::Black,
    CRGB::Black
};
